@@ -32,6 +32,7 @@ License version 3 and version 2.1 along with this program.  If not, see
 
 #include <gtk/gtk.h>
 #include <glib.h>
+#include <atk/atk.h>
 
 #include "client.h"
 #include "menuitem.h"
@@ -505,7 +506,6 @@ menu_item_stop_activating(DbusmenuMenuitem * mi)
 		guint id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(parent),
 		                           data_idle_close_id));
 		if (id > 0) {
-			g_source_remove(id);
 			g_object_set_data(G_OBJECT(parent), data_idle_close_id,
 			                  GINT_TO_POINTER(0));
 			should_close = TRUE;
@@ -574,6 +574,14 @@ close_in_idle (DbusmenuMenuitem * mi)
 	return FALSE;
 }
 
+static void
+cancel_idle_close_id (gpointer data)
+{
+  guint id = GPOINTER_TO_INT(data);
+  if (id > 0)
+  	g_source_remove(id);
+}
+
 static void
 submenu_notify_visible_cb (GtkWidget * menu, GParamSpec * pspec, DbusmenuMenuitem * mi)
 {
@@ -591,8 +599,8 @@ submenu_notify_visible_cb (GtkWidget * menu, GParamSpec * pspec, DbusmenuMenuite
 		                           data_idle_close_id));
 		if (id == 0) {
 			id = g_idle_add((GSourceFunc)close_in_idle, mi);
-			g_object_set_data(G_OBJECT(mi), data_idle_close_id,
-			                  GINT_TO_POINTER(id));
+			g_object_set_data_full(G_OBJECT(mi), data_idle_close_id,
+			                       GINT_TO_POINTER(id), cancel_idle_close_id);
 		}
 	}
 }
@@ -717,6 +725,45 @@ process_disposition (DbusmenuMenuitem * mi, GtkMenuItem * gmi, GVariant * varian
 	return;
 }
 
+/* Process the accessible description */
+static void
+process_a11y_desc (DbusmenuMenuitem * mi, GtkMenuItem * gmi, GVariant * variant, DbusmenuGtkClient * gtkclient)
+{
+	AtkObject * aobj = gtk_widget_get_accessible(GTK_WIDGET(gmi));
+
+	if (aobj == NULL) {
+		return;
+	}
+
+
+	if (variant != NULL) {
+		const gchar * setname = NULL;
+		setname = g_variant_get_string(variant, NULL);
+		atk_object_set_name(aobj, setname);
+	} else {
+	/* The atk docs advise to set the name of the atk object to an empty
+	 * string, but GTK doesn't yet do the same, and setting the name to NULL
+	 * causes tests to fail.
+	 */
+		const gchar * label = NULL;
+		label = dbusmenu_menuitem_property_get(mi, DBUSMENU_MENUITEM_PROP_LABEL);
+
+		if (label != NULL) {
+			gchar * setname = NULL;
+
+			/* We don't want the underscore for mnewmonics */
+			GRegex * regex = g_regex_new ("_", 0, 0, NULL);
+			setname = g_regex_replace_literal (regex, label, -1, 0, "", 0, NULL);
+			g_regex_unref(regex);
+
+			atk_object_set_name(aobj, setname);
+			g_free(setname);
+		}
+	}
+
+	return;
+}
+
 /* Whenever we have a property change on a DbusmenuMenuitem
    we need to be responsive to that. */
 static void
@@ -738,22 +785,15 @@ menu_prop_change_cb (DbusmenuMenuitem * mi, gchar * prop, GVariant * variant, Db
 		process_submenu(mi, gmi, variant, gtkclient);
 	} else if (!g_strcmp0(prop, DBUSMENU_MENUITEM_PROP_DISPOSITION)) {
 		process_disposition(mi, gmi, variant, gtkclient);
+	} else if (!g_strcmp0(prop, DBUSMENU_MENUITEM_PROP_ACCESSIBLE_DESC)) {
+		process_a11y_desc(mi, gmi, variant, gtkclient);
+	} else if (!g_strcmp0(prop, DBUSMENU_MENUITEM_PROP_SHORTCUT)) {
+		refresh_shortcut(gtkclient, mi);
 	}
 
 	return;
 }
 
-/* Special handler for the shortcut changing as we need to have the
-   client for that one to get the accel group. */
-static void
-menu_shortcut_change_cb (DbusmenuMenuitem * mi, gchar * prop, GVariant * value, DbusmenuGtkClient * client)
-{
-	if (!g_strcmp0(prop, DBUSMENU_MENUITEM_PROP_SHORTCUT)) {
-		refresh_shortcut(client, mi);
-	}
-	return;
-}
-
 /* The new menuitem signal only happens if we don't have a type handler
    for the type of the item.  This should be an error condition and we're
    printing out a message. */
@@ -869,7 +909,6 @@ dbusmenu_gtkclient_newitem_base (DbusmenuGtkClient * client, DbusmenuMenuitem *
 
 	/* DbusmenuMenuitem signals */
 	g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_PROPERTY_CHANGED, G_CALLBACK(menu_prop_change_cb), client);
-	g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_PROPERTY_CHANGED, G_CALLBACK(menu_shortcut_change_cb), client);
 	g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_CHILD_REMOVED, G_CALLBACK(delete_child), client);
 	g_signal_connect(G_OBJECT(item), DBUSMENU_MENUITEM_SIGNAL_CHILD_MOVED,   G_CALLBACK(move_child),   client);
 
@@ -883,8 +922,14 @@ dbusmenu_gtkclient_newitem_base (DbusmenuGtkClient * client, DbusmenuMenuitem *
 	process_toggle_state(item, gmi, dbusmenu_menuitem_property_get_variant(item, DBUSMENU_MENUITEM_PROP_TOGGLE_STATE));
 	process_submenu(item, gmi, dbusmenu_menuitem_property_get_variant(item, DBUSMENU_MENUITEM_PROP_CHILD_DISPLAY), client);
 	process_disposition(item, gmi, dbusmenu_menuitem_property_get_variant(item, DBUSMENU_MENUITEM_PROP_DISPOSITION), client);
+	process_a11y_desc(item, gmi, dbusmenu_menuitem_property_get_variant(item, DBUSMENU_MENUITEM_PROP_ACCESSIBLE_DESC), client);
 	refresh_shortcut(client, item);
 
+	const gchar * a11y_desc = dbusmenu_menuitem_property_get(item, DBUSMENU_MENUITEM_PROP_ACCESSIBLE_DESC);
+	if (a11y_desc != NULL) {
+		atk_object_set_name(gtk_widget_get_accessible(GTK_WIDGET(gmi)), a11y_desc);
+	}
+
 	/* Oh, we're a child, let's deal with that */
 	if (parent != NULL) {
 		new_child(parent, item, dbusmenu_menuitem_get_position(item, parent), DBUSMENU_GTKCLIENT(client));
@@ -1112,12 +1157,7 @@ image_property_handle (DbusmenuMenuitem * item, const gchar * property, GVariant
 		return;
 	}
 
-	const gchar * value = NULL;
-	if (variant != NULL) {
-		value = g_variant_get_string(variant, NULL);
-	}
-	
-	if (value == NULL || value[0] == '\0') {
+	if (variant == NULL) {
 		/* This means that we're unsetting a value. */
 		/* Try to use the other one */
 		if (g_strcmp0(property, DBUSMENU_MENUITEM_PROP_ICON_NAME)) {
